/* Syntactic analyzer */

#include <stdbool.h>
#include <stdlib.h>

#include "global_vars.h"
#include "lexical_analyzer.h"
#include "print_helper.h"

#define PRINT_TOKEN 1

/* Syntactic analyzer functions implementation */

enum TokenType tokenType;

static void analyze_TEXT() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == MOT) {
        #if PRINT_TOKEN
        wprint_ctoken(stdout);
        #endif
        if (target != NULL) {
            fputws((const wchar_t*)tokenValue, target);
            fputws(L" ", target);
        }
        scanner();
        analyze_TEXT();
    } else if (tokenType != MOTCLE && tokenType != NPARA && tokenType != SECTION && \
                tokenType != SSECTION && tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        fflush(stderr);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_P() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == NPARA) {
        #if PRINT_TOKEN
        wprint_ctoken(stdout);
        #endif /* PRINT_TOKEN */
        if (target != NULL) {
            fputws(L"<p>\n", target);
        }
        scanner();
        analyze_TEXT();
        if (target != NULL) {
            fputws(L"\n</p>\n", target);
        }
        analyze_P();
        } else if (tokenType != SECTION && tokenType != SSECTION && tokenType != FIN) {
            fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
            fflush(stderr);
            exit(EXIT_FAILURE);
        }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_HEAD() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == MOTCLE && wcscmp(L">Titre", (const wchar_t*)tokenValue) == 0) {
        if (target != NULL) {
            fputws(L"<title>\n", target);
            fputws(L" ", target);
        }
        scanner();
        analyze_TEXT();
        if (target != NULL) {
            fputws(L"\n</title>\n", target);
        }
        if (tokenType == MOTCLE && wcscmp(L">Auteur", (const wchar_t*)tokenValue) == 0) {
            #if PRINT_TOKEN
            wprint_ctoken(stdout);
            #endif /* PRINT_TOKEN */
            fputws(L"<meta name=\"author\" content=\"", target);
            scanner();
            analyze_TEXT();
            fputws(L"\">", target);
        } else if (tokenType != NPARA && tokenType != SECTION && tokenType != FIN) {
            fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
            fflush(stderr);
            exit(EXIT_FAILURE);
        }
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_H1() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == SECTION) {
        #if PRINT_TOKEN
        wprint_ctoken(stdout);
        #endif /* PRINT_TOKEN */
        if (target != NULL) {
            fputws(L"<h1>\n", target);
        }
        scanner();
        analyze_TEXT();
        if (target != NULL) {
            fputws(L"\n</h1>\n", target);
        }
    } else {
        fprintf(stderr, "%s error\n", __func__);
        fflush(stderr);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_H2() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == SSECTION) {
        #if PRINT_TOKEN
        wprint_ctoken(stdout);
        #endif /* PRINT_TOKEN */
        if (target != NULL) {
            fputws(L"<h2>\n", target);
        }
        scanner();
        analyze_TEXT();
        if (target != NULL) {
            fputws(L"\n</h2>\n", target);
        }
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_S2() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == SSECTION) {
        analyze_H2();
        analyze_P();
        analyze_S2();
    } else if (tokenType != SECTION && tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        fflush(stderr);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_S1() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    if (tokenType == SECTION) {
        analyze_H1();
        analyze_P();
        analyze_S2();
        analyze_S1();
    } else if (tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        fflush(stderr);
        exit(EXIT_FAILURE);
    }
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

static void analyze_BODY() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    analyze_P();
    analyze_S1();
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
}

void analyze_AXIOME() {
    #if DEBUG
    fprintf(stdout, "entering %s\n", __func__);
    #endif
    /* print the lexical analysis result */
    #if PRINT_TOKEN
    wprint_ctoken(stdout);
    #endif /* PRINT_TOKEN */
    if (target != NULL) {
        fputws(L"<head>\n", target);
    }
    analyze_HEAD();
    if (target != NULL) {
        fputws(L"\n</head>\n", target);
    }
    if (target != NULL) {
        fputws(L"<body>\n", target);
    }
    analyze_BODY();
    if (target != NULL) {
        fputws(L"\n<body>\n", target);
    }
    #if PRINT_TOKEN
    wprint_ctoken(stdout);
    #endif /* PRINT_TOKEN */
    if (tokenType != FIN) {
        fprintf(stderr, "%s follows error on %s\n", __func__, tokenTypestr[tokenType]);
        fflush(stderr);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "successful syntactic analysis\n");
    #if DEBUG
    fprintf(stdout, "leaving %s\n", __func__);
    #endif
    fflush(stdout);
}
